import { Comparator } from "./comparator";
import { Iterable } from "./iterable";
import { Node } from "./node";
import { Tree } from "./tree";
/**
 * Implements a binary tree structure using a Red/Black tree algorithm.
 *
 */
export declare class BinaryTree<T> extends Tree<T> implements Iterable<T> {
    constructor(arr?: T[], comparator?: Comparator<T>);
    /**
     * Performs a breadth first traversal of the tree and saves it to an array
     * of T type (of the tree).  The array of data is returned.  Don't use this
     * with a large tree.
     * @returns {T[]} an array of all elements in the tree in breadth order
     */
    get breadth(): T[];
    /**
     * @return {number} computes and returns the height of the tree.
     */
    get height(): number;
    /**
     * @return {T[]} the results of an inorder traversal of the tree.  The
     * results are stored in an array and returned.
     */
    get inorder(): T[];
    /**
     * @return {T[]} the results of a postorder traversal of the tree.  The
     * results are stored in an array and returned.
     */
    get postorder(): T[];
    /**
     * @return {T[]} the results of a preorder traversal of the tree.  The
     * results are stored in an array and returned.
     */
    get preorder(): T[];
    /**
     * Peforms a breadth first search against the tree.  Generally this is not
     * the best way to search the tree (use contains to determine if a key is
     * in it).
     * @param data {T} the data element to search for (based on the data type
     * of the tree)
     * @param node {T} the starting node for the search.  This is root by
     * default
     * @return {boolean} true if the item is found in the tree, otherwise
     * false is returned.
     */
    breadthSearch(data: T, node?: Node<T>): boolean;
    /**
     * Performs a typically binary search through the tree.
     * @param data {T} the data element to search for (based on the data type
     * of the tree)
     * @return {boolean} true if the item is found in the tree, otherwise
     * false is returned.
     */
    contains(data: T): boolean;
    /**
     * Searches the tree for an element.  If it is found, then the data element
     * associated with that node is returned (not the node).  When used with a
     * primative type this is not useful as the key and the value found would
     * be the same.  This is helpful when T is a complex object with a custom
     * Comparator.
     * @param key {T} a search key to look for in the tree.
     * @return {T} the full data element within this tree.  If it is not found,
     * then null is returned.
     */
    find(key: T): T;
    /**
     * Internal method that traverses the tree to compute the current height.
     * This is used by the `.height` property
     * @param node {Node<T>} the starting node position to start the height
     * count.
     * @private
     */
    private findHeight;
    private inorderDelegate;
    /**
     * Inserts a data element into the tree.
     * @param data {T} the data element to insert into the tree
     */
    insert(data: T): void;
    private insertDelegate;
    private insertFixUp;
    private newNode;
    private postorderDelegate;
    private preorderDelegate;
    /**
     * Removes the given data value from the tree.
     * @param data {T} the data value to remove
     */
    remove(data: T): T;
    /**
     * Special case function to quickly find and remove the first item in the
     * tree.
     * @return {T} the data element that was first and removed from the tree.
     */
    removeFirst(): T;
    private removeFixUp;
    /**
     * Special case function to quickly find and remove the last item in the
     * tree.
     * @return {T} the data element that was last and removed from the tree.
     */
    removeLast(): T;
    /**
     * Replaces one subtree as a child of its parent with another subtree
     * @param u {Node<T>} parent subtree
     * @param v {Node<T>} child subtree to use in replacement
     * @private
     */
    private transplant;
    /**
     * Searches the tree for a specific node within the tree.
     * @param data {T} the data value to search for in the tree.
     * @return {Node<T>} if the data is found, then the node that holds it is
     * returned.  If it is not found, then nil is returned.
     */
    _findNode(data: T): Node<T>;
    /**
     * Localized left rotation of nodes.  This is a public function but is private
     * by convention (for testing).  Generally not called as part of the api.
     */
    _leftRotate(x: Node<T>): void;
    /**
     * Searches a tree from a given node for the maximum value in that
     * (sub)tree.  Note that the property `.largest` can be used to
     * quickly retrieve the largest value in the tree.  This is really used
     * to recompute the maximum value when it is removed from the tree.
     * @param node {Node<T>} the node location to start the search.  By
     * default this is the root node if no node is given.
     * @return {Node<T>} the largest node in the (sub)tree.
     */
    _maximum(node?: Node<T>): Node<T>;
    /**
     * From a node, searches a tree or subtree for the minimum value in that
     * (sub)tree.  Note that the property `.smallest` can be used to
     * quickly retrieve the smallest value in the tree.  This is really used
     * to recompute the minimum value when it is removed from the tree.
     * @param node {Node<T>} the node location to start the search.  By
     * default this is the root node if no node is given.
     * @return {Node<T>} the smallest node in the (sub)tree.
     */
    _minimum(node?: Node<T>): Node<T>;
    /**
     * Localized right rotation of nodes.  This is a public function but is private
     * by convention (for testing).  Generally not called as part of the api.
     */
    _rightRotate(x: Node<T>): void;
    /**
     * The successor of a node is the node with the smallest key greater than
     * node.data.
     * @param node {Node<T>} the node location to start the search for a
     * successor.
     * @return {Node<T>} a reference to the successor node.
     */
    _successor(node: Node<T>): Node<T>;
    /**
     * Implements an inorder successor algorithm to process each node through
     * an iterator.
     */
    [Symbol.iterator](): Generator<T, void, unknown>;
}
