import { Comparator } from "./comparator";
import { Iterable } from "./iterable";
import { AugmentedNode, Id, Node, NodeData, NodeKeys } from "./node";
import { Tree } from "./tree";
export interface TreeIndex<T> {
    [key: string]: TreeNode<T>;
}
declare type TreeNode<T> = AugmentedNode<T>;
export declare type GeneralTreeItem<T> = TreeNode<T>;
export declare type GeneralTreeFlat<T> = NodeKeys & NodeData<T> & T;
export declare type ToStringCallback<T> = (val: TreeNode<T>) => string;
export declare type WalkCallback<T> = (val: TreeNode<T>) => void;
export interface GeneralTreeOptions<T> {
    sequence?: number;
    testing?: boolean;
    treeData?: Array<TreeNode<T>>;
    useindex?: boolean;
    usesanitize?: boolean;
}
/**
 * Implements a general tree structure
 */
export declare class GeneralTree<T> extends Tree<T> implements Iterable<T> {
    private _dirty;
    private _options;
    private _sequence;
    private _treeIndex;
    constructor(options?: GeneralTreeOptions<T>, comparator?: Comparator<T>);
    /**
     * @return {boolean} true if the tree has been changed before the last walk operation
     */
    get dirty(): boolean;
    /**
     * @return {TreeNode<T>} a reference to the first item in the tree and null if empty.
     */
    get first(): TreeNode<T>;
    /**
     * @return {number} the depth of the tree
     */
    get height(): number;
    /**
     * @return {TreeNode<T>} a reference to the last item in the tree and null if empty.
     */
    get last(): TreeNode<T>;
    /**
     * @return {boolean} returns true if the module is under test, otherwise false
     */
    get testing(): boolean;
    /**
     * @return {Array<TreeNode<T>>} a reference to the current tree array structure
     */
    get treeData(): Array<TreeNode<T>>;
    /**
     * Sets the internal representation of the tree structure array to a new array.
     * Generally this should not be used, but this is exposed for use in another
     * 3rd party library to allow it access to change the struccture (react treeview)
     * @param val {Array<TreeNode<T>>} the new array of nodes to make as the new tree
     */
    set treeData(val: Array<TreeNode<T>>);
    /**
     * @return {TreeIndex<T>} a reference to the that holdes the id/node index relationship
     */
    get treeIndex(): TreeIndex<T>;
    /**
     * @return {boolean} if true, then the index is being used by the tree
     */
    get useindex(): boolean;
    /**
     * @return {boolean} if true, then nodes within the tree will be sanitized when the
     * walk routine is used.  This will ensure that all key fields are present within a
     * node.  This is important when the .treeData function can opverride the current
     * array.  The sanitize is used to ensure tree integrity.
     */
    get usesanitize(): boolean;
    /**
     * Convenience method for adding a node to the index.
     * @param node {TreeNode<T>} - a reference to the node to insert into the index
     */
    private addToIndex;
    /**
     * Resets the tree.  This will clear out the current tree arrayu and all
     * reference pointers to first/last and the current index.
     */
    clear(): void;
    /**
     * Searches through all of the nodes to see if they contain the given
     * search field data.  This works like findByField.  This function
     * wraps that method and checks if it returned anyting.
     * @return {boolean} true if the dat fields are found within the
     * tree, otherwise false.
     */
    contains(dataToFind: T): boolean;
    /**
     * Creates a new node object with the given node properties as
     * a parameter.  The given properties are merged into the newly
     * created node.
     * @param node {GeneralTreeItem} - the node fields to assign to the new
     * node object
     * @return {GeneralTreeItem} a new node instance reference
     */
    createNode(configNode?: GeneralTreeItem<T>): GeneralTreeItem<Node<T>>;
    /**
     * Takes a 1D array, created by the flatten function, and expands it into
     * the tree.  The current tree is replaced by this expanded tree.
     * @param flatNodes {Array<GeneralFlatTree<T>>} - the array of flattened
     * nodes that will be used to expand the tree.
     */
    expand(flatNodes: Array<GeneralTreeFlat<T>>): void;
    /**
     * Performs a breadth search of the tree for a matching id value.  If
     * the item has been seen before, then it is retrieved from an index (if
     * useindex is set to true).
     * @param searchId {Id} the id value to search for in the tree
     * @return {GeneralTreeItem} of the item found otherwise null
     */
    find(searchId: Id): TreeNode<T>;
    /**
     * Searches every node in the tree for data matching the given search critera
     * in the dataToFind parameter.  The comparator defined when the class
     * is instantiated is used to compare the dataToFind against each node in the
     * tree.  Every node that is found is placed into an array and that
     * array is returned to the caller.
     * @param dataToFind {T} - the input data matching the template fields for
     * the tree.
     * @returns {GeneralTreeItem[]} an array of tree nodes that match
     * the search criteria.  If no nodes are found, then an empty array is
     * returned.
     */
    findByField(dataToFind: T): Array<TreeNode<T>>;
    /**
     * Retrieves the list of nodes associated with the given parent key.
     * This is basically get all of the children associated with the
     * given parent ID value.
     * @param parentId {Id} - the parent id key value field to search
     * for.
     * @return {GeneralTreeItem[]} - all of the children associated with
     * the given parent.  If there are no children, then an empty array
     * is returned.
     */
    findByParent(parentId: Id): AugmentedNode<T>[];
    /**
     * Walks through the tree data and flattens it into a 1D array of nodes.
     * This flatten can be reversed using the expand function.  The process
     * removes properties from Node that are not needed once the flatten is
     * complete.  The only fields saved for the flatten are the id, the
     * parent id, and the fields that are part of the template T.
     * @return {Array<GeneralTreeFlat<T>>} an array of nodes representing the
     * tree and its parent/child key relationship.
     */
    flatten(): Array<GeneralTreeFlat<T>>;
    /**
     * Generates a new unique key for a node.  When the testing flag is set
     * to true, then the id is an ordered sequence.  This is done to make
     * the keys predictable when the code is under test.
     * @return {Id} the new key value.
     */
    private getNewKey;
    /**
     * Takes a child array and and id and searches the array for the existence
     * of that id within it.
     * @param searchId {Id} - the id value to use in the search
     * @param children {Array<TreeNode<T>>} - child array to search for an id
     * @return {boolean} true if the id was found in the array, otherise false.
     */
    private isIdInChildren;
    /**
     * Searches the current child list for an id and returns the index location
     * within the child list where it is at.  If it is not found, then the
     * index is -1.
     * @param searchId {Id} - the id value to use in the search
     * @param children {Array<TreeNode<T>>} - child array to search for an id
     * @reutrn {number} the index location of the child within the array. A
     * -1 is returned if not found.
     */
    private indexInChildren;
    /**
     * Inserts a new node into the general tree.
     * @param dataToInsert {TreeNode<T>} - the config information for the node
     * that will be inserted (note that this is NOT the node inserted).  Generally
     * the parentId is used to determine where it will be inserted.
     * @param asFirstChild=true {boolean} - determines what end of the child
     * array will be used when adding the new node to the tree.  If true, then
     * the new node is inserted at the beginning of the array, otherise at the
     * end.  The default behavior is the front.
     * @param validate=false {boolean} - if true, then validate .id values
     * on the input config data to ensure no duplidates.
     * @return {GeneralTreeItem<Node<T>>} a reference to the new node that was
     * inserted into the tree.  On error this is null (with a corresponding
     * warning message).
     */
    insert(dataToInsert: TreeNode<T>, asFirstChild?: boolean, validate?: boolean): GeneralTreeItem<Node<T>>;
    /**
     * Removes a node from the tree.  If the node is a parent with
     * children, then the `deleteWithChildren` flag can be used to
     * stop the delete process if children are present.  This is allowed
     * by default (so deleting a parent deletes its children).  When this
     * flag is true, then the delete would not be allowed until all of the
     * children are deleted first.  The delete operations are expensive
     * because a re-walk of the tree is required to fix indexing and
     * first/last pointers.
     * @param idToRemove {Id} - the id value of the node to remove
     * @param deleteWithChildren=false {boolean} - if true, then a parent
     * with children cannot be deleted until all children are removed.
     */
    remove(idToRemove: Id, deleteWithChildren?: boolean): void;
    private removeFromIndex;
    /**
     * Takes an input node and ensures that it has all key fields.	 It
     * also creates the node key value if one does not exist.
     * @param node {TreeNode<T>} the node to fix
     * @return {TreeNode<T>} a referece back of the node that was sanitized
     */
    private sanitize;
    /**
     * Iterates through all of the nodes and concatenates them into a string.
     * Only prints the relevant key information from the tree.  This also
     * takes a callback function sent by the user to deal with the template
     * data that may be part of the tree nodes.  The internals of the tree
     * can't know the T value(s) while running.  This callback will
     * receive a reference to the node being printed.  The caller can then
     * use this to print the T values it passed into the tree.
     * @param dynamicDataTCallback {ToStringCallback<T>} - a function that can
     * process the tempate data T and return a string representation of it.
     * @return {string} the string representing the keys and T data in the
     * general tree.
     */
    toString(dynamicDataTCallback?: ToStringCallback<T>): string;
    /**
     * Performs an inorder traversal of the current tree.  At each node
     * a callback function is executed and the node being processed
     * is given as a parameter.
     * @param fn {WalkCallback<T>} a callback function invoked on each
     * node as it is encountered.
     */
    walk(fn: WalkCallback<T>): void;
    /**
     * An inorder iterator through the nodes of the general tree.
     */
    [Symbol.iterator](): Generator<AugmentedNode<T>, void, unknown>;
}
export default GeneralTree;
