import { Collection } from "./collection";
import { Comparator } from "./comparator";
import { Iterable } from "./iterable";
import { Node } from "./node";
/**
 * A doubly linked list structure.
 *
 */
export declare class List<T> extends Collection<T> implements Iterable<T> {
    static readonly FRONT: number;
    static readonly BACK: number;
    constructor(arr?: T[], comparator?: Comparator<T>);
    /**
     * @return {T[]} an inorder array of all elements in the list
     */
    get array(): T[];
    /**
     * @return {T[]} a reversed array of all elements in the list
     */
    get reverse(): T[];
    /**
     * Retrieves the data from the list at a given position.  If the position
     * is outside of the list, then null is returned.  This is an O(N)
     * operation for each call (it looks for each position from the start
     * of th list).
     * @param pos {number} the position to retrieve from the list
     * @return {T} the data associated with this position within the list
     */
    at(pos: number): T;
    /**
     * Searches the list for an element.  If it is found, then the data element
     * associated with that node is returned (not the node).  When used with a
     * primative type this is not useful as the key and the value found would
     * be the same.  This is helpful when T is a complex object with a custom
     * Comparator.
     * @param key {T} a search key to look for in the list.
     * @return {T} the full data element within this list.  If it is not found,
     * then null is returned.
     */
    find(key: T): T;
    /**
     * Inserts a data value into a linked list.  The default operation is to
     * insert into the end of the list.
     * @param data {T} the data item to insert into the list
     * @param idx {number} the index position where the item will be inserted.
     * this number be List.FRONT, List.BACK, or any other number.
     */
    insert(data: T, idx?: number): void;
    /**
     * Removes an element from the list.
     * @param data {T} the data item removed from the list
     * @param [idx] {number} the position index to remove from the list
     * @return {T} the data element that was deleted
     */
    remove(data: T, idx?: number): T;
    /**
     * Iterates through the list and find the node associated with a key
     * value T.
     * @param data {T} the key value or index to search for in the list.
     * @return {Node<T>} a reference to the node that was found.  If a node
     * is not found, then null is returned.
     */
    _getNodeByValue(data: T): Node<T>;
    /**
     * Moves to a position within the list and finds the node associated
     * with that index.
     * @param idx {number} the location within the list to find
     * @return {Node<T>} a reference to a node a the given position.  If the
     * node is not found, then null is returned.
     */
    _getNodeByIndex(idx: number): Node<T>;
    /**
     * An inorder iterator through the list of current values.
     */
    [Symbol.iterator](): Generator<T, void, unknown>;
}
