import { BinaryTree } from "./binarytree";
export interface Offset {
    [key: number]: number;
}
export interface Priority<T> {
    priority: string;
    data?: T;
}
/**
 * A simple priority queue.  Items are inserted into a queue structure with an
 * associatd priority number.  The lower the number, the higher the priority.
 * Items with a higher priority are chosen first in the queue.
 *
 * Priority values are numbers > 0.  If a negative number is given, then it
 * will be given a priority of 0.
 *
 * The implementation is a Red/Black tree.  That makes getting the lowest
 * value from a set of values an O(1) operation; we always know the smallest
 * value inserted into the tree.
 *
 */
export declare class PriorityQueue<T> extends BinaryTree<Priority<T>> {
    private _offsets;
    constructor();
    /**
     * @return {T[]} an array of values in the queue in priority order.  This
     * does not change the queue.
     */
    get array(): T[];
    /**
     * Clears the queue and resets the internals.
     */
    clear(): void;
    /**
     * Retrieves the highest priority item from the queue and returns it.
     * @return {T} the data with the highest priority.  If no data is available
     * then null is returned;
     */
    dequeue(): T;
    /**
     * Removes all items from the queue in order and returns them as an
     * array of data values.
     * @returns {T[]} a list of queue items as an array.
     */
    drain(): T[];
    /**
     * Adds an item to the queue in priority order.  Items of the same priority
     * are sorted by insertion order.  The lower number represents higher
     * priority.
     * @param data {T} the data element to add to the queue
     * @param priority {number} a number >= 0 that sets the priority of the
     * item in the queue.
     */
    enqueue(data: T, priority: number): void;
}
